#
# BRM Plugin for BigBrotherBot(B3) (www.bigbrotherbot.com)
# Feature additions to work with Brothers Realism Mod www.codbrothers.co.uk/brm
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#

__version__ = '1.0.0'
__author__  = 'Brother-Robbo'

import b3, re
import b3.events

#--------------------------------------------------------------------------------------------------
class BrmPlugin(b3.plugin.Plugin):
  _adminPlugin = None

  def startup(self):
    """\
    Initialize plugin settings
    """

    # get the admin plugin so we can register commands
    self._adminPlugin = self.console.getPlugin('admin')
    if not self._adminPlugin:
      # something is wrong, can't start without admin plugin
      self.error('Could not find admin plugin')
      return False
    
    # register our commands
    if 'commands' in self.config.sections():
      for cmd in self.config.options('commands'):
        level = self.config.get('commands', cmd)
        sp = cmd.split('-')
        alias = None
        if len(sp) == 2:
          cmd, alias = sp

        func = self.getCmd(cmd)
        if func:
          self._adminPlugin.registerCommand(self, cmd, level, func, alias)

    self.debug('Started')


  def getCmd(self, cmd):
    cmd = 'cmd_%s' % cmd
    if hasattr(self, cmd):
      func = getattr(self, cmd)
      return func

    return None


  def onEvent(self, event):
    """\
    Handle intercepted events
    """


#----------- commands -----#
  def cmd_adminspec(self, data, client, cmd=None):                         
    """\                                                                   
    sets self to view all players
    Safe to use without the brm prefix
    """           
    self.console.setCvar('b3_bro_adminspec','%s' % client.cid )

    return True
    
  def cmd_brmclanmode(self, data, client, cmd=None):                         
    """\                                                                   
    <set mode> (on / off). Leave blank to get status
    Safe to use without the brm prefix
    """           
    cm = self.console.getCvar('bro_clanmode')[0]
    
    if not data:
      if int(cm) == 1:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Clan Mode: ^2Enabled')
      else:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Clan Mode: ^3Disabled')
    else:
      if int(cm) == 1 and (data == '0' or data == 'off' or data == 'disabled'):
        self.console.setCvar('bro_clanmode','0' )
        self.console.say('|^4BRM^7| Clan Mode Will be disabled next round')
      elif int(cm) == 0 and (data == '1' or data == 'on' or data == 'enabled'): 
        self.console.setCvar('bro_clanmode','1')
        self.console.say('|^4BRM^7| Clan Mode Will be enabled next round')
      elif int(cm) == 1 and (data == '1' or data == 'on' or data == 'enabled'):
        client.message('|^4BRM^7| Clan mode is already enabled')
      elif int(cm) == 0 and (data == '0' or data == 'off' or data == 'disabled'):
        client.message('|^4BRM^7| Clan mode is already disabled')  
      else:
        client.message('^1An error has occured (invalid parameter)')  
      
    return True
    
  def cmd_brmclanteam(self, data, client, cmd=None):                         
    """\                                                                   
    <team> (allies, axis, both) sets the allowed clan team in clanmode.
    Leave blank to get status
    Safe to use without the brm prefix
    """           
    
    if not data:
      team = self.console.getCvar('bro_clanteam')[0]
      if int(team) == 1:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Clan Team: ^2Allies')
      elif int(team) == 2:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Clan Team: ^2Axis')
      else:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Clan Team: ^2Both')
    else:
      cm = self.console.getCvar('bro_clanmode')[0]
      
      if data == '0' or data == 'both':
        self.console.setCvar('bro_clanteam','0' )
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Clan Team set to ^3Both')
        else:
          client.message('|^4BRM^7| Clan Team set to ^3Both ^7 - Clan Mode currently ^3Disabled')  
          
      elif data == '1' or data == 'allies': 
        self.console.setCvar('bro_clanteam','1')
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Clan Team set to ^3Allies')
        else:
          client.message('|^4BRM^7| Clan Team set to ^3Allies ^7 - Clan Mode currently ^3Disabled')
      elif data == '2' or data == 'axis': 
        self.console.setCvar('bro_clanteam','2')
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Clan Team set to ^3Axis')
        else:
          client.message('|^4BRM^7| Clan Team set to ^3Axis ^7 - Clan Mode currently ^3Disabled')

      else:
        client.message('^1An error has occured (invalid parameter)')  
      
    return True
    
  def cmd_brmpubteam(self, data, client, cmd=None):                         
    """\                                                                   
    <team> (allies, axis, both) sets the allowed public team in clanmode.
    Leave blank to get status
    Safe to use without the brm prefix
    """           
        
    if not data:
      team = self.console.getCvar('bro_pubteam')[0]
      
      if int(team) == 1:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Public Team: ^2Allies')
      elif int(team) == 2:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Public Team: ^2Axis')
      else:
        cmd.sayLoudOrPM(client, '|^4BRM^7| Public Team: ^2Both')
    else:
      cm = self.console.getCvar('bro_clanmode')[0]
      if data == '0' or data == 'both':
        self.console.setCvar('bro_pubteam','0' )
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Public Team set to ^3Both')
        else:
          client.message('|^4BRM^7| Public Team set to ^3Both ^7 - Clan Mode currently ^3Disabled')
      elif data == '1' or data == 'allies': 
        self.console.setCvar('bro_pubteam','1')
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Public Team set to ^3Allies')
        else:
          client.message('|^4BRM^7| Public Team set to ^3Allies ^7 - Clan Mode currently ^3Disabled')
      elif data == '2' or data == 'axis': 
        self.console.setCvar('bro_pubteam','2')
        
        if int(cm) == 1:
          self.console.say('|^4BRM^7| Public Team set to ^3Axis')
        else:
          client.message('|^4BRM^7| Public Team set to ^3Axis ^7 - Clan Mode currently ^3Disabled')

      else:
        client.message('^1An error has occured (invalid parameter)')  
      
    return True
    