#include maps\mp\_utility;
#include maps\mp\gametypes\_hud_util;

/*
	Retrieval
	Attackers objective: Bomb one of 2 positions
	Defenders objective: Defend these 2 positions / Defuse planted bombs
	Round ends:	When one team is eliminated, bomb explodes, bomb is defused, or roundlength time is reached
	Map ends:	When one team reaches the score limit, or time limit or round limit is reached
	Respawning:	Players remain dead for the round and will respawn at the beginning of the next round

	Level requirements
	------------------
		Allied Spawnpoints:
			classname		mp_sd_spawn_attacker
			Allied players spawn from these. Place at least 16 of these relatively close together.

		Axis Spawnpoints:
			classname		mp_sd_spawn_defender
			Axis players spawn from these. Place at least 16 of these relatively close together.

		Spectator Spawnpoints:
			classname		mp_global_intermission
			Spectators spawn from these and intermission is viewed from these positions.
			Atleast one is required, any more and they are randomly chosen between.

		Retrieval objective locations:
			classname					trigger_multiple
			targetname					bombzone
			script_gameobjectname		bombzone
			script_bombmode_original	<if defined this bombzone will be used in the original bomb mode>
			script_bombmode_single		<if defined this bombzone will be used in the single bomb mode>
			script_bombmode_dual		<if defined this bombzone will be used in the dual bomb mode>
			script_team					Set to allies or axis. This is used to set which team a bombzone is used by in dual bomb mode.
			script_label				Set to A or B. This sets the letter shown on the compass in original mode.
			This is a volume of space in which the bomb can planted. Must contain an origin brush.

		retrieval objective:
			classname				trigger_lookat
			targetname				bombtrigger
			script_gameobjectname	bombzone
			This should be a 16x16 unit trigger with an origin brush placed so that it's center lies on the bottom plane of the trigger.
			Must be in the level somewhere. This is the trigger that is used when defusing a bomb.
			It gets moved to the position of the planted bomb model.

	Level script requirements
	-------------------------
		Team Definitions:
			game["allies"] = "marines";
			game["axis"] = "japanese";
			This sets the nationalities of the teams. Allies can be american, british, or russian. Axis can be german.

			game["attackers"] = "allies";
			game["defenders"] = "axis";
			This sets which team is attacking and which team is defending. Attackers plant the bombs. Defenders protect the targets.

		If using minefields or exploders:
			maps\mp\_load::main();

	Optional level script settings
	------------------------------
		Soldier Type and Variation:
			game["american_soldiertype"] = "normandy";
			game["german_soldiertype"] = "normandy";
			This sets what character models are used for each nationality on a particular map.

			Valid settings:
				american_soldiertype	normandy
				british_soldiertype		normandy, africa
				russian_soldiertype		coats, padded
				german_soldiertype		normandy, africa, winterlight, winterdark

*/

/*QUAKED mp_sd_spawn_attacker (0.0 1.0 0.0) (-16 -16 0) (16 16 72)
Attacking players spawn randomly at one of these positions at the beginning of a round.*/

/*QUAKED mp_sd_spawn_defender (1.0 0.0 0.0) (-16 -16 0) (16 16 72)
Defending players spawn randomly at one of these positions at the beginning of a round.*/

main()
{
	if(getdvar("mapname") == "mp_background")
		return;
	

	maps\mp\gametypes\_globallogic::init();
	maps\mp\gametypes\_callbacksetup::SetupCallbacks();
	maps\mp\gametypes\_globallogic::SetupCallbacks();

	
	maps\mp\gametypes\_globallogic::registerRoundSwitchDvar( level.gameType, 1, 0, 9 );
	maps\mp\gametypes\_globallogic::registerTimeLimitDvar( level.gameType, 5, 0, 1440 );
	maps\mp\gametypes\_globallogic::registerScoreLimitDvar( level.gameType, 5, 0, 500 );
	maps\mp\gametypes\_globallogic::registerRoundLimitDvar( level.gameType, 0, 0, 12 );
	maps\mp\gametypes\_globallogic::registerNumLivesDvar( level.gameType, 1, 0, 15 );
	
	if(getdvar("scr_ret_objectiveglow") == "") setdvar("scr_ret_objectiveglow",1);
	
	level.objectiveGlow = getdvarint("scr_ret_objectiveglow");
		
	level.teamBased = true;
	level.overrideTeamScore = true;
	level.onPrecacheGameType = ::onPrecacheGameType;
	level.onStartGameType = ::onStartGameType;
	level.onSpawnPlayer = ::onSpawnPlayer;
	level.onSpawnPlayerUnified = ::onSpawnPlayerUnified;
	level.playerSpawnedCB = ::ret_playerSpawnedCB;
	level.onPlayerKilled = ::onPlayerKilled;
	level.onDeadEvent = ::onDeadEvent;
	level.onOneLeftEvent = ::onOneLeftEvent;
	level.onTimeLimit = ::onTimeLimit;
	level.onRoundSwitch = ::onRoundSwitch;

	level.getTeamKillPenalty = ::ret_getTeamKillPenalty;
	level.getTeamKillScore = ::ret_getTeamKillScore;
	
	level.endGameOnScoreLimit = false;
	
	game["dialog"]["gametype"] = "searchdestroy";
	game["dialog"]["offense_obj"] = "obj_destroy";
	game["dialog"]["defense_obj"] = "obj_defend";
	game["dialog"]["sudden_death"] = "suddendeath_boost";

}


onPrecacheGameType()
{
//	game["objective_model"] = "static_peleliu_crate_rifle_grenades";
	game["objective_model"] = "static_peleliu_radio_backpack";
	game["objective_model_glow"] = "static_peleliu_radio_back_obj";
	game["obj_dropped_sound"] = "flag_drop_plr";
//	game["obj_recovered_sound"] = "flag_pickup_plr";
	
	game["obj_recovered_sound"] = "mp_war_objective_taken";
	
	precacheModel(game["objective_model"]);
	
	precacheModel(game["objective_model_glow"]);

	precacheShader("waypoint_bomb");
	precacheShader("compass_waypoint_bomb");
	precacheShader("specialty_fireproof");
	precacheShader("mortar_map_targetarea");
	precacheShader("objective");
	precacheShader("waypoint_target");
	precacheShader("waypoint_target_a");
	precacheShader("waypoint_target_b");
	precacheShader("waypoint_defend");
	precacheShader("waypoint_defend_a");
	precacheShader("waypoint_defend_b");
	precacheShader("waypoint_defuse");
	precacheShader("waypoint_defuse_a");
	precacheShader("waypoint_defuse_b");
	precacheShader("compass_waypoint_target");
	precacheShader("compass_waypoint_target_a");
	precacheShader("compass_waypoint_defend");
	precacheShader("compass_waypoint_defend_a");
	precacheShader("compass_waypoint_defuse");
	precacheShader("compass_waypoint_defuse_a");
	
	precacheString( &"RET_OBJECTIVE_RECOVERED_BY" );
	precacheString( &"RET_OBJECTIVE_DROPPED_BY" );
	precacheString( &"RET_PRESS_TO_PICKUP" );
	
	
   precacheShader("waypoint_capture");
	precacheShader("compass_waypoint_capture");
	precacheShader("waypoint_defend");
	precacheShader("compass_waypoint_defend");


	
	
	

}
ret_getTeamKillPenalty( eInflictor, attacker, sMeansOfDeath, sWeapon )
{
	teamkill_penalty = maps\mp\gametypes\_globallogic::default_getTeamKillPenalty( eInflictor, attacker, sMeansOfDeath, sWeapon );
	
	return teamkill_penalty;
}

ret_getTeamKillScore( eInflictor, attacker, sMeansOfDeath, sWeapon )
{
	teamkill_score = maps\mp\gametypes\_rank::getScoreInfoValue( "kill" );
	
	
	return int(teamkill_score);
}


onRoundSwitch()
{
	if ( !isdefined( game["switchedsides"] ) )
		game["switchedsides"] = false;
	
	if ( game["teamScores"]["allies"] == level.scorelimit - 1 && game["teamScores"]["axis"] == level.scorelimit - 1 )
	{
		// overtime! team that's ahead in kills gets to defend.
		aheadTeam = getBetterTeam();
		if ( aheadTeam != game["defenders"] )
		{
			game["switchedsides"] = !game["switchedsides"];
		}
		else
		{
			level.halftimeSubCaption = "";
		}
		level.halftimeType = "overtime";
	}
	else
	{
		level.halftimeType = "halftime";
		game["switchedsides"] = !game["switchedsides"];
	}
}

getBetterTeam()
{
	kills["allies"] = 0;
	kills["axis"] = 0;
	deaths["allies"] = 0;
	deaths["axis"] = 0;
	
	for ( i = 0; i < level.players.size; i++ )
	{
		player = level.players[i];
		team = player.pers["team"];
		if ( isDefined( team ) && (team == "allies" || team == "axis") )
		{
			kills[ team ] += player.kills;
			deaths[ team ] += player.deaths;
		}
	}
	
	if ( kills["allies"] > kills["axis"] )
		return "allies";
	else if ( kills["axis"] > kills["allies"] )
		return "axis";
	
	// same number of kills

	if ( deaths["allies"] < deaths["axis"] )
		return "allies";
	else if ( deaths["axis"] < deaths["allies"] )
		return "axis";
	
	// same number of deaths
	
	if ( randomint(2) == 0 )
		return "allies";
	return "axis";
}

onStartGameType()
{
	if ( !isDefined( game["switchedsides"] ) )
		game["switchedsides"] = false;
	
	if ( game["switchedsides"] )
	{
		oldAttackers = game["attackers"];
		oldDefenders = game["defenders"];
		game["attackers"] = oldDefenders;
		game["defenders"] = oldAttackers;
	}
	
	setClientNameMode( "auto_change" );
	
   flagBaseFX = [];
	flagBaseFX["marines"] = "misc/ui_flagbase_blue";
	flagBaseFX["japanese"] = "misc/ui_flagbase_blue";
	flagBaseFX["german"] = "misc/ui_flagbase_blue";
	flagBaseFX["russian"] = "misc/ui_flagbase_blue";
	/*
	flagBaseFX["japanese"] = "misc/ui_flagbase_red";
	flagBaseFX["german"] = "misc/ui_flagbase_gold";
	flagBaseFX["russian"] = "misc/ui_flagbase_orange";
	*/
	level.captureZoneFXid[ "allies" ] = loadfx( flagBaseFX[ game[ "allies" ] ] );
	level.captureZoneFXid[ "axis"   ] = loadfx( flagBaseFX[ game[ "axis"   ] ] );
	
	game["strings"]["objective_captured"] = &"RET_OBJECTIVE_RETRIEVED";
	
	precacheString( game["strings"]["objective_captured"] );
		
	maps\mp\gametypes\_globallogic::setObjectiveText( game["attackers"], &"RET_RET_ATTACKER" );
	maps\mp\gametypes\_globallogic::setObjectiveText( game["defenders"], &"RET_RET_DEFENDER" );

   maps\mp\gametypes\_globallogic::setObjectiveScoreText( game["attackers"], &"RET_RET_ATTACKER_SCORE" );
   maps\mp\gametypes\_globallogic::setObjectiveScoreText( game["defenders"], &"RET_RET_DEFENDER_SCORE" );

	maps\mp\gametypes\_globallogic::setObjectiveHintText( game["attackers"], &"RET_RET_ATTACKER_HINT" );
	maps\mp\gametypes\_globallogic::setObjectiveHintText( game["defenders"], &"RET_RET_DEFENDER_HINT" );

	level.spawnMins = ( 0, 0, 0 );
	level.spawnMaxs = ( 0, 0, 0 );	
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints( "mp_sd_spawn_attacker" );
	maps\mp\gametypes\_spawnlogic::placeSpawnPoints( "mp_sd_spawn_defender" );
	maps\mp\gametypes\_spawning::remove_unused_spawn_entities();
	
		level.mapCenter = maps\mp\gametypes\_spawnlogic::findBoxCenter( level.spawnMins, level.spawnMaxs );
	setMapCenter( level.mapCenter );
	
	allowed[0] = "sd";
	allowed[1] = "blocker";
	allowed[2] = "bombzone";
	maps\mp\gametypes\_gameobjects::main(allowed);

	// now that the game objects have been deleted place the influencers
	maps\mp\gametypes\_spawning::create_map_placed_influencers();

	level.spawn_axis_start = maps\mp\gametypes\_spawnlogic::getSpawnpointArray( "mp_sd_spawn_defender" );
	level.spawn_allies_start = maps\mp\gametypes\_spawnlogic::getSpawnpointArray( "mp_sd_spawn_attacker" );

	/*
	maps\mp\gametypes\_rank::registerScoreInfo( "win", 2 );
	maps\mp\gametypes\_rank::registerScoreInfo( "loss", 1 );
	maps\mp\gametypes\_rank::registerScoreInfo( "tie", 1.5 );	
	maps\mp\gametypes\_rank::registerScoreInfo( "kill", 50 );
	maps\mp\gametypes\_rank::registerScoreInfo( "headshot", 50 );
	maps\mp\gametypes\_rank::registerScoreInfo( "capture", 100 );
	maps\mp\gametypes\_rank::registerScoreInfo( "assist_75", 25 );
	maps\mp\gametypes\_rank::registerScoreInfo( "assist_50", 25 );
	maps\mp\gametypes\_rank::registerScoreInfo( "assist_25", 25 );
	maps\mp\gametypes\_rank::registerScoreInfo( "assist", 25 ); */
   
   maps\mp\gametypes\_sdutils::modifyScoreWeightings("ret");
   
	thread updateGametypeDvars();
	
	if(isdefined(game["preMatch"]) && game["preMatch"] == false )
      thread retObjective();
}


onSpawnPlayerUnified()
{
/*
	if ( !isDefined( self.carryIcon ) && self.pers["team"] == game["attackers"] )
	{

      self.carryIcon = createIcon( "hud_suitcase_bomb", 50, 50 );
      self.carryIcon setPoint( "CENTER", "CENTER", 220, 140 );
      self.carryIcon.alpha = 0.75;
	}
	*/
	maps\mp\gametypes\_spawning::onSpawnPlayer_Unified();
}


onSpawnPlayer()
{
	if(self.pers["team"] == game["attackers"])
		spawnPointName = "mp_sd_spawn_attacker";
	else
		spawnPointName = "mp_sd_spawn_defender";
/*
	if ( !isDefined( self.carryIcon ) && self.pers["team"] == game["attackers"])
	{
      self.carryIcon = createIcon( "hud_suitcase_bomb", 50, 50 );
      self.carryIcon setPoint( "CENTER", "CENTER", 220, 140 );
      self.carryIcon.alpha = 0.75;
	}
*/
	spawnPoints = maps\mp\gametypes\_spawnlogic::getSpawnpointArray( spawnPointName );
	assert( spawnPoints.size );
	spawnpoint = maps\mp\gametypes\_spawnlogic::getSpawnpoint_Random( spawnPoints );

	self spawn( spawnpoint.origin, spawnpoint.angles );
}


ret_playerSpawnedCB()
{
	level notify ( "spawned_player" );
}


onPlayerKilled(eInflictor, attacker, iDamage, sMeansOfDeath, sWeapon, vDir, sHitLoc, psOffsetTime, deathAnimDuration)
{
	thread checkAllowSpectating();
}


checkAllowSpectating()
{
	wait ( 0.05 );
	
	update = false;
	if ( !level.aliveCount[ game["attackers"] ] )
	{
		level.spectateOverride[game["attackers"]].allowEnemySpectate = 1;
		update = true;
	}
	if ( !level.aliveCount[ game["defenders"] ] )
	{
		level.spectateOverride[game["defenders"]].allowEnemySpectate = 1;
		update = true;
	}
	if ( update )
		maps\mp\gametypes\_spectating::updateSpectateSettings();
}


ret_endGame( winningTeam, endReasonText )
{
	if ( isdefined( winningTeam ) )
		[[level._setTeamScore]]( winningTeam, [[level._getTeamScore]]( winningTeam ) + 1 );
	
	thread maps\mp\gametypes\_globallogic::endGame( winningTeam, endReasonText );
}


onDeadEvent( team )
{
	
	if ( team == "all" )
	{
			ret_endGame( game["defenders"], game["strings"][game["attackers"]+"_eliminated"] );
	}
	else if ( team == game["attackers"] )
	{
		ret_endGame( game["defenders"], game["strings"][game["attackers"]+"_eliminated"] );
	}
	else if ( team == game["defenders"] )
	{
		ret_endGame( game["attackers"], game["strings"][game["defenders"]+"_eliminated"] );
	}
}


onOneLeftEvent( team )
{

	//if ( team == game["attackers"] )
	warnLastPlayer( team );
}


onTimeLimit()
{
	if ( level.teamBased )
		ret_endGame( game["defenders"], game["strings"]["time_limit_reached"] );
	else
		ret_endGame( undefined, game["strings"]["time_limit_reached"] );
}


warnLastPlayer( team )
{
	if ( !isdefined( level.warnedLastPlayer ) )
		level.warnedLastPlayer = [];
	
	if ( isDefined( level.warnedLastPlayer[team] ) )
		return;
		
	level.warnedLastPlayer[team] = true;

	players = level.players;
	for ( i = 0; i < players.size; i++ )
	{
		player = players[i];

		if ( isDefined( player.pers["team"] ) && player.pers["team"] == team && isdefined( player.pers["class"] ) )
		{
			if ( player.sessionstate == "playing" && !player.afk )
				break;
		}
	}
	
	if ( i == players.size )
		return;
	
	players[i] thread giveLastAttackerWarning();
}


giveLastAttackerWarning()
{
	self endon("death");
	self endon("disconnect");
	
	fullHealthTime = 0;
	interval = .05;
	
	while(1)
	{
		if ( self.health != self.maxhealth )
			fullHealthTime = 0;
		else
			fullHealthTime += interval;
		
		wait interval;
		
		if (self.health == self.maxhealth && fullHealthTime >= 3)
			break;
	}
	
	//self iprintlnbold(&"MP_YOU_ARE_THE_ONLY_REMAINING_PLAYER");
	//self maps\mp\gametypes\_globallogic::leaderDialogOnPlayer( "last_alive" );
	self maps\mp\gametypes\_globallogic::leaderDialogOnPlayer( "sudden_death" );
	
	self maps\mp\gametypes\_missions::lastManSD();
}


updateGametypeDvars()
{
	level.teamKillPenaltyMultiplier = dvarFloatValue( "teamkillpenalty", 2, 0, 10 );
	level.teamKillScoreMultiplier = dvarFloatValue( "teamkillscore", 4, 0, 40 );
}


retObjective()
{
   level.retObjectiveCaptured = false;
   
   visuals[0] = getEnt( "sd_bomb", "targetname" );
	if ( !isDefined( visuals[0] ) )
	{
		maps\mp\_utility::error("No sd_bomb script_model found in map.");
		return;
	}
	
	if(game["bromode"])
      visuals[0] delete();
   else     
      visuals[0] setModel(game["objective_model"]);
	
	goaltrigger = getEnt( "sd_bomb_pickup_trig", "targetname" );
	if ( !isDefined( goaltrigger ) )
	{
		maps\mp\_utility::error("No sd_bomb_pickup_trig trigger found in map.");
		return;
	}
		
	trigger = getEnt( "bombtrigger", "targetname" );
	if ( !isDefined( trigger ) )
	{
		maps\mp\_utility::error("No bombtrigger trigger found in map.");
		return;
	}
   
   bombZones = getEntArray( "bombzone", "targetname" );
   
   if(game["bromode"])
   {
      level.retObjectMove = [];
      
      for(i=0; i < bombzones.size; i++)
      {
         visuals2 = [];
         visuals2[0] = spawn("script_model",bombzones[i].origin);
         visuals2[0].angle = bombzones[i].angles;
         visuals2[0] setModel(game["objective_model"]);
         
         trig = spawn("trigger_radius", visuals2[0].origin, 0, 10, 50);
         
         level.retObjectMove[i] = maps\mp\gametypes\_gameobjects::createCarryObject( game["attackers"], trig, visuals2, (0,0,32) );
        // level.retObjectMove[i] = maps\mp\gametypes\_gameobjects::createCarryObject( game["attackers"], bombzones[i], visuals2, (0,0,32) );
      

         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::allowUse( "none" );
         
         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::set2DIcon( "friendly", "compass_waypoint_capture" );
         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::set3DIcon( "friendly", "waypoint_capture" );
         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::set2DIcon( "enemy", "compass_waypoint_defend" );
         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::set3DIcon( "enemy", "waypoint_defend" );
         level.retObjectMove[i] maps\mp\gametypes\_gameobjects::setVisibleTeam( "any" );
         
         traceStart = visuals2[0].origin + (0,0,32);
         
     //    level.retObjectMove[i] = spawnStruct();
         
    //     level.retObjectMove[i].visuals = visuals2;
         
     //    level.retObjectMove[i] thread glowModel();

         level.retObjectMove[i] thread dropObjectiveitem(tracestart);
      }
   
   }
   else
   {
      bombsiteChance = randomInt(bombZones.size);
      
      site = bombZones[bombsiteChance];
      
      traceStart = site.origin + (0,0,32);
      
      visuals[0].angles = site.angles;
      
      level.retObject = maps\mp\gametypes\_gameobjects::createCarryObject( game["attackers"], trigger, visuals, (0,0,32) );

      level.retObject maps\mp\gametypes\_gameobjects::allowUse( "friendly" );
               
      level.retObject maps\mp\gametypes\_gameobjects::set2DIcon( "friendly", "compass_waypoint_capture" );
      level.retObject maps\mp\gametypes\_gameobjects::set3DIcon( "friendly", "waypoint_capture" );
      level.retObject maps\mp\gametypes\_gameobjects::set2DIcon( "enemy", "compass_waypoint_defend" );
      level.retObject maps\mp\gametypes\_gameobjects::set3DIcon( "enemy", "waypoint_defend" );
      level.retObject maps\mp\gametypes\_gameobjects::setVisibleTeam( "any" );
      
      level.retObject maps\mp\gametypes\_gameobjects::setUseHintText( &"RET_PRESS_TO_PICKUP" );
            
      level.retObject maps\mp\gametypes\_gameobjects::setCarryIcon( "specialty_fireproof" ); 
      
      level.retObject.allowWeapons = true; // add dvar option
      
      level.retObject.onPickup = ::onPickup;
      
      level.retObject.onDrop = ::onDrop;
      
      level.retObject.onCantUse = ::onCantUse;
      
      level.retObject thread dropObjectiveitem(tracestart);
   }
	
//	if(!game["bromode"])
//	{

      for(y=0; y < bombZones.size; y++)
      {
           
         vis = getEntArray( bombZones[y].target, "targetname" );
         
         
               
         defuse = getent(vis[0].target, "targetname");
            
         if(!getdvarint("scr_ret_keepsdvisuals_" + level.script))
         {
            
            brushmodels = getentarray("script_brushmodel", "classname");
            for(j=0;j<brushmodels.size;j++)
            {
           //    if (isDefined(brushmodels[j].script_gameobjectname) && brushmodels[j].script_gameobjectname == "bombzone" && distance(brushmodels[j].origin, bombZones[y].origin) < 64)	
               if (isDefined(brushmodels[j].script_gameobjectname) && brushmodels[j].script_gameobjectname == "bombzone")	
                  brushmodels[j] delete();
            }
        
            des = getentarray("exploder", "targetname");
            for(j=0;j<des.size;j++)
            {
               if (isDefined(des[j].script_exploder) && des[j].script_exploder == vis[0].script_exploder)
                  des[j] delete();
            }
            
            for(j=0;j<vis.size;j++)
            {
               vis[j] delete();
            }
         
         }  
         defuse delete();
         
       //  if(!game["bromode"])
         bombZones[y] delete();
       
         
      }
   
 //  }
   
   visuals = [];
	
	entityTeam = game["attackers"];

	level.retCaptureZone = maps\mp\gametypes\_gameobjects::createUseObject( entityTeam, goaltrigger, visuals, (0,0,50) );
	
	if(game["bromode"])
      level.retCaptureZone maps\mp\gametypes\_gameobjects::allowUse( "none" );
	else
      level.retCaptureZone maps\mp\gametypes\_gameobjects::allowUse( "friendly" );
	
	level.retCaptureZone maps\mp\gametypes\_gameobjects::setUseTime( 0 );
	//level.retCaptureZone maps\mp\gametypes\_gameobjects::setUseText( &"MP_CAPTURING_FLAG" );
	level.retCaptureZone maps\mp\gametypes\_gameobjects::setVisibleTeam( "any" );
	level.retCaptureZone maps\mp\gametypes\_gameobjects::set2DIcon( "friendly", "mortar_map_targetarea" );
	
	level.retCaptureZone maps\mp\gametypes\_gameobjects::set2DIcon( "enemy", "mortar_map_targetarea" );
//	level.retCaptureZone maps\mp\gametypes\_gameobjects::set3DIcon( "enemy", level.iconTakenEnemy3D );

   if(game["bromode"])
	{
      level.retCaptureZone maps\mp\gametypes\_gameobjects::set3DIcon( "friendly", "mortar_map_targetarea" );
      level.retCaptureZone maps\mp\gametypes\_gameobjects::set3DIcon( "enemy", "mortar_map_targetarea" );
   }
	
	level.retCaptureZone maps\mp\gametypes\_gameobjects::setKeyObject( level.retObject );
	level.retCaptureZone.onUse = ::onCapture;
	
	traceStart = goaltrigger.origin + (0,0,32);
//	traceEnd = goaltrigger.origin + (0,0,-32);
	
	thread loadZoneFX(traceStart);
	
}

onCantUse( player )
{
	player iPrintLnBold( "MP_CANT_PLANT_WITHOUT_BOMB" );
}

loadZoneFX(pos)
{
   wait 0.1;
   trace = bulletTrace( pos, pos - (0,0,64), false, undefined );

	upangles = vectorToAngles( trace["normal"] );
	level.retCaptureZone.baseeffectforward = anglesToForward( upangles );
	level.retCaptureZone.baseeffectright = anglesToRight( upangles );
	
	level.retCaptureZone.baseeffectpos = trace["position"] + (0,0,1);
	
   level.retCaptureZone thread resetBaseEffect();
}

resetBaseEffect()
{
	// dont spawn first frame
	wait (0.1);
	
	if ( isdefined( self.baseeffect ) )
		self.baseeffect delete();
	
	team = self maps\mp\gametypes\_gameobjects::getOwnerTeam();
	
	if ( team != "axis" && team != "allies" )
		return;
	
	fxid = level.captureZoneFXid[ team ];

	self.baseeffect = spawnFx( fxid, self.baseeffectpos, self.baseeffectforward, self.baseeffectright );
//	self.baseeffect = spawnFx( fxid, self.baseeffectpos);
	
	triggerFx( self.baseeffect );
}

dropObjectiveitem(tracestart)
{
   wait 0.05;
 //  wait 7;
   
   trace = bulletTrace( traceStart, traceStart - (0,0,200), false, self.visuals[0] );

//	upangles = vectorToAngles( trace["normal"] );
   self.visuals[0].origin = trace["position"];
   self.trigger.origin = trace["position"]; 
   self.trigger.baseOrigin = trace["position"]; 
   self.curOrigin = trace["position"];
   self.visuals[0].baseOrigin = trace["position"];
   
   if(level.objectiveGlow || game["bromode"])
   {
      self.glow = spawn("script_model", self.visuals[0].origin);
      self.glow.angles = self.visuals[0].angles;
      self.glow setmodel(game["objective_model_glow"]);
      self.glow show();
   }
   
   
  // self glowModel();
   maps\mp\gametypes\_gameobjects::updateWorldIcons();
	maps\mp\gametypes\_gameobjects::updateCompassIcons();
	
}

glowModel()
{
   wait 0.05;
   if(level.objectiveGlow || game["bromode"])
   {
      self.glow = spawn("script_model", self.visuals[0].origin);
      self.glow.angles = self.visuals[0].angles;
      self.glow setmodel(game["objective_model_glow"]);
      self.glow show();
   }
}

onCapture( player )
{
  // iprintln(player.name + " has captured the objective!");
   iprintln(&"RET_OBJECTIVE_RECOVERED_BY",player);
   team = player.pers["team"];
	enemyTeam = getOtherTeam( team );
	
 //  printAndSoundOnEveryone( team, enemyTeam, &"RET_OBJECTIVE_RECOVERED_BY", &"RET_OBJECTIVE_RECOVERED_BY", "mp_obj_captured", "mp_obj_captured", player );
   thread playSoundOnPlayers( "mx_CTF_score"+"_"+level.teamPrefix[team] );
   lpselfnum = player getEntityNumber();
	lpGuid = player getGuid();
	logPrint("OC;" + lpGuid + ";" + lpselfnum + ";" + player.name + "\n");
	
	level.retObject maps\mp\gametypes\_gameobjects::disableObject();
	
	maps\mp\gametypes\_globallogic::givePlayerScore( "capture", player );
	
	setGameEndTime( 0 );
	
	wait 3;
	
	ret_endGame( game["attackers"], game["strings"]["objective_captured"] );
}


onDrop( player )
{
   if(isDefined(level.retObject.glow))
   {
      level.retObject.glow.origin = level.retObject.visuals[0].origin;
      level.retObject.glow.angles = level.retObject.visuals[0].angles;
      level.retObject.glow show();
   }
   
		if ( isDefined( player ) && isDefined( player.name ) )
			printOnTeamArg( &"RET_OBJECTIVE_DROPPED_BY", game["attackers"], player );

		if ( isDefined( player ) )
		 	player logString( "objective dropped" );
		 else
		 	logString( "objective dropped" );
	
   if ( isDefined( player ) )
      player notify( "event_ended" );

	self maps\mp\gametypes\_gameobjects::set3DIcon( "friendly", "waypoint_capture" );
	self maps\mp\gametypes\_gameobjects::set2DIcon( "friendly", "compass_waypoint_capture" );
	
//	if(getdvar("bro_noTinkles") == "1") 
      maps\mp\_utility::playSoundOnPlayers( game["obj_dropped_sound"], game["attackers"] );
}


onPickup( player )
{
	
	if(isDefined(level.retObject.glow))
      level.retObject.glow hide();
   
	self maps\mp\gametypes\_gameobjects::set3DIcon( "friendly", "waypoint_capture" );
	self maps\mp\gametypes\_gameobjects::set2DIcon( "friendly", "compass_waypoint_bomb" );
	self maps\mp\gametypes\_gameobjects::setVisibleTeam( "friendly" );

		if ( isDefined( player ) && isDefined( player.name ) )
			player thread pickupNotify();
			
	if(getdvar("bro_noTinkles") != "1")	
      thread playSoundOnPlayers( "mx_SD_pickup"+"_"+level.teamPrefix[player.pers["team"]], player.pers["team"] );

		maps\mp\gametypes\_globallogic::leaderDialog( "bomb_taken", player.pers["team"] );
		player logString( "objective taken" );
	
	if(getdvar("bro_noTinkles") != "1") 
      maps\mp\_utility::playSoundOnPlayers( game["obj_recovered_sound"], game["attackers"] );
   
   curWeap = player getCurrentWeapon();
   player giveWeapon("radio_drop_mp");
   player SetActionSlot(2,"weapon","radio_drop_mp");
   player switchToWeapon(curWeap);
}

pickupNotify()
{
   wait 0.7;
   if ( isDefined( self ) && isDefined( self.name ) )
      printOnTeamArg( &"RET_OBJECTIVE_RECOVERED_BY", self.pers["team"], self );

}



onReset()
{
}


playSoundinSpace( alias, origin )
{
	org = spawn( "script_origin", origin );
	org.origin = origin;
	org playSound( alias  );
	wait 10; // MP doesn't have "sounddone" notifies =(
	org delete();
}


